/**
 * This file Copyright (c) 2018 Magnolia International
 * Ltd.  (http://www.magnolia-cms.com). All rights reserved.
 *
 *
 * This file is dual-licensed under both the Magnolia
 * Network Agreement and the GNU General Public License.
 * You may elect to use one or the other of these licenses.
 *
 * This file is distributed in the hope that it will be
 * useful, but AS-IS and WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE, or NONINFRINGEMENT.
 * Redistribution, except as permitted by whichever of the GPL
 * or MNA you select, is prohibited.
 *
 * 1. For the GPL license (GPL), you can redistribute and/or
 * modify this file under the terms of the GNU General
 * Public License, Version 3, as published by the Free Software
 * Foundation.  You should have received a copy of the GNU
 * General Public License, Version 3 along with this program;
 * if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * 2. For the Magnolia Network Agreement (MNA), this file
 * and the accompanying materials are made available under the
 * terms of the MNA which accompanies this distribution, and
 * is available at http://www.magnolia-cms.com/mna.html
 *
 * Any modifications to this file must keep this entire header
 * intact.
 *
 */
package info.magnolia.ui.vaadin.ckeditor;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.vaadin.alump.ckeditor.CKEditorConfig;

import com.google.gson.Gson;

/**
 * Magnolia 6 - Vaadin 8 extension of CKEditorConfig in order to define a more
 * coherent toolbar API and configure custom plugins.
 */
public class MagnoliaCKEditorConfig extends CKEditorConfig {

    private List<String> events = new ArrayList<>();

    private Map<String, String> externalPlugins = new HashMap<>();

    /**
     * Adds list of toolbar groups. Groups are placed in one
     * row if space allows.
     */
    public void addToolbarLine(List<ToolbarGroup> toolbars) {
        Gson gson = new Gson();
        String json = gson.toJson(toolbars);
        addCustomToolbarLine(json.substring(1, json.length() - 1));
    }

    /**
     * Add CKEditor event name that server side will listen.
     * Only event names added to configuration will be listened.
     *
     * @param eventName This must match in client side. e.g: editor.fire('eventName')
     */
    public void addListenedEvent(String eventName) {
        events.add(eventName);
    }

    public String[] getListenedEvents() {
        return events.toArray(new String[0]);
    }

    public void addExternalPlugin(String pluginName, String source) {
        externalPlugins.put(pluginName, source);
    }

    public Map<String, String> getExternalPlugins() {
        return externalPlugins;
    }

    @Override
    public void addExtraConfig(String extraConfigName, String extraConfigValue) {
        super.addExtraConfig(extraConfigName, extraConfigValue);
    }

    @Override
    public synchronized void addToRemovePlugins(String pluginName) {
        super.addToRemovePlugins(pluginName);
    }

    @Override
    public synchronized void addToExtraPlugins(String pluginName) {
        super.addToExtraPlugins(pluginName);
    }

    @Override
    public void setBaseFloatZIndex(int zIndex) {
        super.setBaseFloatZIndex(zIndex);
    }

    @Override
    public void setResizeEnabled(boolean enabled) {
        super.setResizeEnabled(enabled);
    }

    /**
     * Represents a CKEditor toolbar.
     */
    public static class ToolbarGroup {

        private String name;

        private List<String> items = new ArrayList<>();

        public ToolbarGroup(String groupname, String... toolbarbuttons) {
            this.name = groupname;
            Collections.addAll(items, toolbarbuttons);
        }

        public String getName() {
            return name;
        }

        public void addItem(String item) {
            items.add(item);
        }
    }

    public static List<ToolbarGroup> defaultToolbar() {
        List<ToolbarGroup> toolbars = new ArrayList<>();
        toolbars.add(new ToolbarGroup("basicstyles", "Bold", "Italic", "Underline", "SpecialChar"));
        toolbars.add(new ToolbarGroup("paragraph", "NumberedList", "BulletedList", "JustifyLeft", "JustifyCenter", "JustifyRight", "JustifyBlock", "Image", "Table"));
        toolbars.add(new ToolbarGroup("links", "Link", "InternalLink", "DamLink", "Unlink"));
        toolbars.add(new ToolbarGroup("styles", "Font", "FontSize", "TextColor"));
        toolbars.add(new ToolbarGroup("clipboard", "Cut", "Copy", "Paste", "PasteText", "PasteFromWord"));
        toolbars.add(new ToolbarGroup("undo", "Undo", "Redo"));
        toolbars.add(new ToolbarGroup("tools", "Source"));
        return toolbars;
    }
}