/**
 * This file Copyright (c) 2018 Magnolia International
 * Ltd.  (http://www.magnolia-cms.com). All rights reserved.
 *
 *
 * This file is dual-licensed under both the Magnolia
 * Network Agreement and the GNU General Public License.
 * You may elect to use one or the other of these licenses.
 *
 * This file is distributed in the hope that it will be
 * useful, but AS-IS and WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE, or NONINFRINGEMENT.
 * Redistribution, except as permitted by whichever of the GPL
 * or MNA you select, is prohibited.
 *
 * 1. For the GPL license (GPL), you can redistribute and/or
 * modify this file under the terms of the GNU General
 * Public License, Version 3, as published by the Free Software
 * Foundation.  You should have received a copy of the GNU
 * General Public License, Version 3 along with this program;
 * if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * 2. For the Magnolia Network Agreement (MNA), this file
 * and the accompanying materials are made available under the
 * terms of the MNA which accompanies this distribution, and
 * is available at http://www.magnolia-cms.com/mna.html
 *
 * Any modifications to this file must keep this entire header
 * intact.
 *
 */
package info.magnolia.ui.vaadin.gwt.client.extension;

import info.magnolia.ui.vaadin.extension.FocusFieldCaption;
import info.magnolia.ui.vaadin.gwt.client.jquerywrapper.JQueryWrapper;

import com.google.gwt.core.client.JsArray;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.Style;
import com.google.gwt.event.dom.client.BlurEvent;
import com.google.gwt.event.dom.client.FocusEvent;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.EventListener;
import com.google.gwt.user.client.ui.Widget;
import com.vaadin.client.ComponentConnector;
import com.vaadin.client.ServerConnector;
import com.vaadin.client.extensions.AbstractExtensionConnector;
import com.vaadin.shared.ui.Connect;

/**
 * Client-side connector for the {@link FocusFieldCaption}.
 */
@Connect(FocusFieldCaption.class)
public class FocusFieldCaptionConnector extends AbstractExtensionConnector {

    @Override
    protected void extend(ServerConnector target) {

        final Widget widget = ((ComponentConnector) target).getWidget();

        target.addStateChangeHandler(handler -> {
            if (handler.getConnector().getState().enabled) {

                final JsArray<com.google.gwt.user.client.Element> elements = JQueryWrapper
                                                                                .select(widget)
                                                                                .find(".v-formlayout-captioncell .v-caption")
                                                                                .get();
                for (int i = 0; i < elements.length(); i++) {
                    final Element element = elements.get(i);
                    final Element spanLabel = element.getFirstChildElement();
                    final Element controlContainer = DOM.getElementById(spanLabel.getAttribute("for"));
                    if (controlContainer == null) {
                        continue;
                    }

                    // allow div container to fire onfocus & onblur events
                    controlContainer.setAttribute("tabindex", "-1");
                    controlContainer.getStyle().setOutlineStyle(Style.OutlineStyle.NONE);

                    // add event listener to container div (this works with composite components like basic upload or
                    // asset upload
                    final EventListener containerListener = DOM.getEventListener(controlContainer);
                    DOM.sinkEvents(controlContainer, Event.ONFOCUS | Event.ONBLUR);
                    DOM.setEventListener(controlContainer, event -> {
                        int eventCode = event.getTypeInt();
                        if (Event.ONFOCUS == eventCode) {
                            handleFocusAndBlurEvent(controlContainer, element, true);
                        } else if (Event.ONBLUR == eventCode) {
                            handleFocusAndBlurEvent(controlContainer, element, false);
                        } else {
                            if (containerListener != null) {
                                containerListener.onBrowserEvent(event);
                            }
                        }
                    });

                    // handling extra inputs, textarea(s) which can be nested under multiple level
                    final JsArray<com.google.gwt.user.client.Element> nestedInputs = JQueryWrapper
                                                                                        .select(DOM.asOld(controlContainer))
                                                                                        .find("textarea, input[type]:not([type=search]):not([type=url]):not([type=hidden])")
                                                                                        .get();
                    for (int j = 0; j < nestedInputs.length(); j++) {
                        final com.google.gwt.user.client.Element input = nestedInputs.get(j);
                        final EventListener listener = DOM.getEventListener(input);
                        if (listener instanceof Widget) {
                            ((Widget) listener).addHandler(event -> handleFocusAndBlurEvent(input, element, true), FocusEvent.getType());
                            ((Widget) listener).addHandler(event -> handleFocusAndBlurEvent(input, element, false), BlurEvent.getType());
                        }
                    }
                }
            }
        });
    }

    private void handleFocusAndBlurEvent(Element control, Element labelOfControl, boolean focus) {
        if (control.hasClassName("v-readonly") || control.hasClassName("v-label")) {
            return;
        }
        if (focus) {
            labelOfControl.addClassName("v-caption-focus");
        } else {
            labelOfControl.removeClassName("v-caption-focus");
        }
    }

}
