/**
 * This file Copyright (c) 2017-2018 Magnolia International
 * Ltd.  (http://www.magnolia-cms.com). All rights reserved.
 *
 *
 * This file is dual-licensed under both the Magnolia
 * Network Agreement and the GNU General Public License.
 * You may elect to use one or the other of these licenses.
 *
 * This file is distributed in the hope that it will be
 * useful, but AS-IS and WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE, or NONINFRINGEMENT.
 * Redistribution, except as permitted by whichever of the GPL
 * or MNA you select, is prohibited.
 *
 * 1. For the GPL license (GPL), you can redistribute and/or
 * modify this file under the terms of the GNU General
 * Public License, Version 3, as published by the Free Software
 * Foundation.  You should have received a copy of the GNU
 * General Public License, Version 3 along with this program;
 * if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * 2. For the Magnolia Network Agreement (MNA), this file
 * and the accompanying materials are made available under the
 * terms of the MNA which accompanies this distribution, and
 * is available at http://www.magnolia-cms.com/mna.html
 *
 * Any modifications to this file must keep this entire header
 * intact.
 *
 */
package info.magnolia.ui.vaadin.gwt.client.extension;

import info.magnolia.ui.vaadin.extension.MessageBubble;
import info.magnolia.ui.vaadin.gwt.client.form.formsection.widget.InlineMessageWidget;
import info.magnolia.ui.vaadin.gwt.client.icon.widget.IconWidget;
import info.magnolia.ui.vaadin.gwt.client.shared.messagebubble.MessageBubbleAlignment;
import info.magnolia.ui.vaadin.gwt.client.shared.messagebubble.MessageBubbleType;

import com.google.gwt.dom.client.Style;
import com.google.gwt.user.client.ui.Widget;
import com.vaadin.client.ServerConnector;
import com.vaadin.client.communication.StateChangeEvent;
import com.vaadin.client.extensions.AbstractExtensionConnector;
import com.vaadin.client.ui.AbstractComponentConnector;
import com.vaadin.shared.ui.Connect;

/**
 * Client-side connector for {@link MessageBubble}.
 */
@Connect(MessageBubble.class)
public class MessageBubbleConnector extends AbstractExtensionConnector {
    private MessageBubbleWidget messageBubbleWidget;
    private Widget widget;

    @Override
    protected void extend(ServerConnector target) {
        widget = ((AbstractComponentConnector) target).getWidget();
    }

    @Override
    public void onStateChanged(StateChangeEvent stateChangeEvent) {
        super.onStateChanged(stateChangeEvent);
        MessageBubbleState state = getState();
        removeMessageBubble();
        if (state.title != null && state.message != null && widget != null) {
            messageBubbleWidget = createMessageBubbleWidget(state.messageType, state.alignment);
            if (messageBubbleWidget == null) return;
            messageBubbleWidget.setMessage(state.title, state.message);
            widget.getElement().getParentElement().getParentElement().getParentElement().getStyle().setVerticalAlign(Style.VerticalAlign.TOP);
            widget.getElement().getParentElement().appendChild(messageBubbleWidget.getElement());
        }
    }

    @Override
    public MessageBubbleState getState() {
        return (MessageBubbleState) super.getState();
    }

    @Override
    public void onUnregister() {
        super.onUnregister();
        removeMessageBubble();
    }

    private void removeMessageBubble() {
        if (messageBubbleWidget != null) {
            messageBubbleWidget.getElement().removeFromParent();
        }
    }

    private abstract class MessageBubbleWidget extends InlineMessageWidget {

        private final IconWidget compositeIcon = new IconWidget();

        public MessageBubbleWidget(String primaryStyleName) {
            super();
            setStyleName(primaryStyleName + "-inline");
            messageWrapper.addClassName(primaryStyleName);

            triangleSVG.addStyleName("triangle");

            compositeIcon.setStyleName("composite-icon");
            compositeIcon.addStyleName("message-bubble-icon");
            compositeIcon.getElement().getStyle().setProperty("lineHeight", "1.4");
            messageWrapper.replaceChild(compositeIcon.getElement(), icon);
            messageEl.getStyle().setDisplay(Style.Display.BLOCK);
            messageEl.getStyle().setMarginLeft(30, Style.Unit.PX);
        }

        @Override
        protected void applyStyles() {
        }

        public IconWidget getCompositeIcon() {
            return compositeIcon;
        }

        public void addSubIcon(IconWidget subIcon) {
            compositeIcon.getElement().appendChild(subIcon.getElement());
            subIcon.getElement().getStyle().setProperty("lineHeight", "1.4");
        }

        public void setTriangleAlignment(MessageBubbleAlignment alignment) {
            triangleSVG.getElement().addClassName(alignment.getCssClass());
        }

        public void setMessage(String title, String message) {
            final StringBuilder sb = new StringBuilder();
            sb.append("<b>").append(title).append("</b>");
            sb.append("<div>").append(message).append("</div>");
            messageEl.setInnerHTML(sb.toString());
        }
    }

    private MessageBubbleWidget createMessageBubbleWidget(MessageBubbleType messageType, MessageBubbleAlignment alignment) {
        MessageBubbleWidget widget = null;

        switch (messageType) {
        case ERROR:
            widget = new ErrorBubbleWidget();
            break;
        case WARNING:
            widget = new WarningBubbleWidget();
            break;
        case INFO:
            widget = new InfoBubbleWidget();
            break;
        case HELP:
            widget = new HelpBubbleWidget();
            break;
        }
        if (widget != null) {
            widget.setTriangleAlignment(alignment);
        }
        return widget;
    }

    private class ErrorBubbleWidget extends MessageBubbleWidget {
        public ErrorBubbleWidget() {
            super("error-bubble");
            triangleSVG.addStyleName("red");
            getCompositeIcon().addStyleName("error-icon");
            addSubIcon(new IconWidget("shape-triangle", "#fff"));
            addSubIcon(new IconWidget("error-mark", "#b00"));
        }
    }

    private class WarningBubbleWidget extends MessageBubbleWidget {
        public WarningBubbleWidget() {
            super("warning-bubble");
            triangleSVG.addStyleName("yellow");
            getCompositeIcon().addStyleName("warning-icon");
            addSubIcon(new IconWidget("shape-circle-plus"));
            addSubIcon(new IconWidget("shape-circle"));
            addSubIcon(new IconWidget("warning-mark"));
        }
    }

    private class InfoBubbleWidget extends MessageBubbleWidget {
        public InfoBubbleWidget() {
            super("info-bubble");
            triangleSVG.addStyleName("blue");
            getCompositeIcon().addStyleName("info-icon");
            addSubIcon(new IconWidget("shape-circle-plus"));
            addSubIcon(new IconWidget("shape-circle"));
            addSubIcon(new IconWidget("info_mark"));
        }
    }

    private class HelpBubbleWidget extends MessageBubbleWidget {
        public HelpBubbleWidget() {
            super("help-bubble");
            triangleSVG.addStyleName("blue");
            getCompositeIcon().addStyleName("help-icon");
            addSubIcon(new IconWidget("shape-circle-plus"));
            addSubIcon(new IconWidget("shape-circle"));
            addSubIcon(new IconWidget("help-mark"));
        }
    }
}
