/**
 * This file Copyright (c) 2018 Magnolia International
 * Ltd.  (http://www.magnolia-cms.com). All rights reserved.
 *
 *
 * This file is dual-licensed under both the Magnolia
 * Network Agreement and the GNU General Public License.
 * You may elect to use one or the other of these licenses.
 *
 * This file is distributed in the hope that it will be
 * useful, but AS-IS and WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE, or NONINFRINGEMENT.
 * Redistribution, except as permitted by whichever of the GPL
 * or MNA you select, is prohibited.
 *
 * 1. For the GPL license (GPL), you can redistribute and/or
 * modify this file under the terms of the GNU General
 * Public License, Version 3, as published by the Free Software
 * Foundation.  You should have received a copy of the GNU
 * General Public License, Version 3 along with this program;
 * if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * 2. For the Magnolia Network Agreement (MNA), this file
 * and the accompanying materials are made available under the
 * terms of the MNA which accompanies this distribution, and
 * is available at http://www.magnolia-cms.com/mna.html
 *
 * Any modifications to this file must keep this entire header
 * intact.
 *
 */
package info.magnolia.ui.vaadin.layout;

import info.magnolia.ui.vaadin.gwt.client.layout.lazylayout.rpc.LazyLayoutClientRpc;
import info.magnolia.ui.vaadin.gwt.client.layout.lazylayout.rpc.LazyLayoutServerRpc;
import info.magnolia.ui.vaadin.gwt.shared.Range;
import info.magnolia.ui.vaadin.layout.LazyLayout.DataProviderKeyMapper;

/**
 * An implementation of {@link LazyLayoutServerRpc} that hosts code common to the two implementations.
 *
 */
public class LazyLayoutServerRpcImpl implements LazyLayoutServerRpc {

    private DataProviderKeyMapper mapper;
    private LazyLayoutClientRpc clientRpc;
    private LazyLayout lazyLayout;

    public LazyLayoutServerRpcImpl(DataProviderKeyMapper mapper, LazyLayoutClientRpc clientRpc, LazyLayout lazyLayout) {
        this.mapper = mapper;
        this.clientRpc = clientRpc;
        this.lazyLayout = lazyLayout;
    }

    public void loadElements(int startFrom, int length, int cachedFirst, int cachedLast) {
        final Range rangeToLoad = Range.withLength(startFrom, length);
        final Range cachedRange = Range.between(cachedFirst, cachedLast);
        final Range activeRange = cachedRange.isEmpty() ? rangeToLoad : rangeToLoad.combineWith(cachedRange);

        mapper.setActiveRange(activeRange);
        clientRpc.addElements(lazyLayout.fetchElements(rangeToLoad), rangeToLoad.getStart());
        lazyLayout.updateSelectionIndices();
    }

    public void onElementsSelected(int index, boolean isMetaKeyPressed, boolean isShiftKeyPressed) {
        lazyLayout.handleSelectionAtIndex(index, isMetaKeyPressed || isShiftKeyPressed);
        lazyLayout.fireSelectionChange();
    }

    public void onElementDoubleClicked(int index) {
        final Object item = mapper.itemAtIndex(index);
        if (item != null) {
            lazyLayout.onElementDoubleClicked(item);
        }
    }

    public void onElementRightClicked(int index, int clickX, int clickY) {
        final Object item = mapper.itemAtIndex(index);
        if (item != null) {
            lazyLayout.onElementRightClicked(item, clickX, clickY);
        }
    }

    public void updateOffset(int currentElementOffset) {
        lazyLayout.getState(false).offset = currentElementOffset;
    }

    public void setScaleRatio(float ratio) {
        lazyLayout.getState(false).scaleRatio = ratio;
    }
}
