/**
 * This file Copyright (c) 2013-2018 Magnolia International
 * Ltd.  (http://www.magnolia-cms.com). All rights reserved.
 *
 *
 * This file is dual-licensed under both the Magnolia
 * Network Agreement and the GNU General Public License.
 * You may elect to use one or the other of these licenses.
 *
 * This file is distributed in the hope that it will be
 * useful, but AS-IS and WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE, or NONINFRINGEMENT.
 * Redistribution, except as permitted by whichever of the GPL
 * or MNA you select, is prohibited.
 *
 * 1. For the GPL license (GPL), you can redistribute and/or
 * modify this file under the terms of the GNU General
 * Public License, Version 3, as published by the Free Software
 * Foundation.  You should have received a copy of the GNU
 * General Public License, Version 3 along with this program;
 * if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * 2. For the Magnolia Network Agreement (MNA), this file
 * and the accompanying materials are made available under the
 * terms of the MNA which accompanies this distribution, and
 * is available at http://www.magnolia-cms.com/mna.html
 *
 * Any modifications to this file must keep this entire header
 * intact.
 *
 */
package info.magnolia.ui.vaadin.layout;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.vaadin.shared.ui.ContentMode;
import com.vaadin.ui.Component;
import com.vaadin.ui.CssLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.VerticalLayout;

/**
 * The SmallAppLayout, offering space for a header component and multiple sections stacked vertically.
 */
public class SmallAppLayout extends VerticalLayout {

    private final CssLayout sectionsLayout = new CssLayout();

    private final CssLayout descriptionLayout = new CssLayout();
    private final Label description = new Label();

    public SmallAppLayout() {
        addStyleName("smallapp");
        setSizeFull();
        setSpacing(false);
        setMargin(false);

        description.addStyleName("smallapp-description");
        description.setContentMode(ContentMode.HTML);
        descriptionLayout.addStyleName("smallapp-description-layout");
        descriptionLayout.setVisible(false);
        descriptionLayout.addComponent(description);

        sectionsLayout.addStyleName("smallapp-sections");
        sectionsLayout.setSizeFull();

        addComponents(descriptionLayout, sectionsLayout);
        setExpandRatio(descriptionLayout, 0f);
        setExpandRatio(sectionsLayout, 1f);
    }

    @Override
    public String getDescription() {
        return description.getValue();
    }

    @Override
    public void setDescription(String description) {
        this.description.setValue(description);
        this.descriptionLayout.setVisible(StringUtils.isNoneBlank(description));
    }

    public List<Component> getSections() {
        Iterator<Component> it = sectionsLayout.iterator();
        List<Component> sections = new ArrayList<>();
        while (it.hasNext()) {
            sections.add(it.next());
        }
        return sections;
    }

    /**
     * Add new section with title into small app layout.
     */
    public void addSection(Component section, String sectionTitle) {
        section.setCaption(sectionTitle);
        addSection(section);
    }

    /**
     * Add new section without a title into small app layout.
     */
    public void addSection(Component section) {
        section.addStyleName("smallapp-section");
        sectionsLayout.addComponent(section);
    }

    public void removeSection(Component section) {
        sectionsLayout.removeComponent(section);
    }
}